//---------------------------------------------------------------------------


#pragma hdrstop

#include "CDecisionLayer.h"
#include <Dialogs.hpp>
#pragma package(smart_init)

//---------------------------------------------------------------------------
CDecisionLayer::CDecisionLayer ()
{
}

//---------------------------------------------------------------------------
// podpora pre loadovanie neuronovych sieti
CDecisionLayer::CDecisionLayer (const AnsiString classString, const AnsiString searchPath)
{
TStringList* temp = new TStringList();
try
  {
  m_ClassString = classString;
  temp->DelimitedText = classString;
  for (int i = 0; i < temp->Count; i++)
    {
    m_LikelihoodDisField.push_back();
    for (int j = i+1; j < temp->Count; j++)
      {
      AnsiString s = searchPath + "\\" + temp->Strings[i]+ "-" + temp->Strings[j] + ".nne";
      CDecisionVertex *v;
      try
        {
        v = new CNeuralNetwork (s);
        m_DecisionVertexField.push_back(v);
        }
      catch (...)
        {
        ShowMessage ("Couldn't open " + s);
        }
      }
    }
  }
catch (...)
  {
  temp->~TStringList();
  }
}

//---------------------------------------------------------------------------

CDecisionLayer::CDecisionLayer (const DecisionClass decisionClass1, const DecisionClass decisionClass2, const int border)
{
CDecisionVertex *v = new CLengthDecision (decisionClass1, decisionClass2, 0, border);
m_DecisionVertexField.push_back(v);
m_LikelihoodDisField.resize(2);
m_ClassString = "";
for (size_t i = 0; i < decisionClass1.size(); i++)
  m_ClassString += decisionClass1[i];

m_ClassString += " ";

for (size_t i = 0; i < decisionClass2.size(); i++)
  m_ClassString += decisionClass2[i];

}

//---------------------------------------------------------------------------

double CDecisionLayer::Mult (vector<double> src, int omit1, int omit2)
{
double result = 1;
for (size_t i = 0; i < src.size(); i++)
  {
  if ((i != size_t(omit1)) && (i != size_t(omit2))) result *= src[i];
  }
return result;
}

//---------------------------------------------------------------------------

void CDecisionLayer::EvaluateLayer (const MFCCVectors input, BOOL useAprox)
{
int k = 0;
if (useAprox)
  for (size_t i = 0; i < m_LikelihoodDisField.size(); i++)
    for (size_t j = i + 1 ; j < m_LikelihoodDisField.size(); j++)
      {
      LikelihoodDistribution temp = m_DecisionVertexField[k]->ComputeLikelihood(input);

      if (i == 0)
        {
        m_LikelihoodDisField[j] = temp.likelihood2;
        if (j == 1) m_LikelihoodDisField[i] = temp.likelihood1;
        else m_LikelihoodDisField[i] += temp.likelihood1;
        }
      else
        {
        m_LikelihoodDisField[i] += temp.likelihood1;
        m_LikelihoodDisField[j] += temp.likelihood2;
        }
      k++;
      }
else
  {
  vector< vector<double> > tempLikelihoodDisField;
  tempLikelihoodDisField.resize(m_LikelihoodDisField.size());
  for (size_t i = 0; i < m_LikelihoodDisField.size(); i++)
    tempLikelihoodDisField[i].resize(m_LikelihoodDisField.size());

  for (size_t i = 0; i < m_LikelihoodDisField.size(); i++)
    for (size_t j = i + 1 ; j < m_LikelihoodDisField.size(); j++)
      {
      LikelihoodDistribution temp = m_DecisionVertexField[k]->ComputeLikelihood(input);
      tempLikelihoodDisField[i][j] = temp.likelihood1;
      tempLikelihoodDisField[j][i] = temp.likelihood2;
      k++;
      /*if (i == 0)
        {
        tempLikelihoodDisField[j] = temp.likelihood2;
        m_LikelihoodDisField[j] = temp.likelihood2;
        if (j == 1)
          {
          tempLikelihoodDisField[i] = temp.likelihood1;
          m_LikelihoodDisField[i] = temp.likelihood1;
          }
       else tempLikelihoodDisField[i] *= temp.likelihood1;
        }
      else
        {
        tempLikelihoodDisField[i] *= temp.likelihood1;
        tempLikelihoodDisField[j] *= temp.likelihood2;
        }
       */
      }

  for (size_t i = 0; i < m_LikelihoodDisField.size(); i++)
    {
    double tempDouble = Mult (tempLikelihoodDisField[i], i);
    double tempDouble2 = 0;
    for (size_t j = 0 ; j < m_LikelihoodDisField.size(); j++)
      tempDouble2 += Mult (tempLikelihoodDisField[i], i, j);

    tempDouble2 -=  m_LikelihoodDisField.size() * (m_LikelihoodDisField.size() - 2) * tempDouble;
    
    if (tempDouble2 != 0) m_LikelihoodDisField[i] = tempDouble / tempDouble2;
    else  m_LikelihoodDisField[i] = 0;
    }
  }

MultiplyLikelihoodDisField(1 / (double) (m_LikelihoodDisField.size() - 1));

int likelihood = 0;
for (size_t i = 0; i < m_LikelihoodDisField.size(); i++)
  if (likelihood <= m_LikelihoodDisField[i])
    {
    likelihood = m_LikelihoodDisField[i];
    m_MaxProbIndex = i;
    }
}

//---------------------------------------------------------------------------

void CDecisionLayer::AddToLines(TStrings *lines, int classIndex)
{
TStringList *temp = new TStringList();
temp->DelimitedText = m_ClassString;

for (int i = 0; i < (int)m_LikelihoodDisField.size(); i++)
  if ((i == classIndex) || ( classIndex == -1))
    {
    lines->Add(temp->Strings[i]);
    lines->Add(FloatToStr(m_LikelihoodDisField[i]));
    }
temp->~TStringList();
}


void CDecisionLayer::MultiplyLikelihoodDisField (double value)
{

for (size_t i = 0; i < m_LikelihoodDisField.size(); i++)
  m_LikelihoodDisField[i] *= value; 
}
