//---------------------------------------------------------------------------


#pragma hdrstop

#include "CDecisionVertex.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

//---------------------------------------------------------------------------

double CDecisionVertex::Distance(MFCCVector from, MFCCVector to)
  {
  if (from.size() != to.size()) ShowMessage("Error in CDecisionVertex::Distance. Vectors with different dimension.");
  double result = 0;
  for (unsigned int i=0; i<from.size(); i++)
    {
    result += (from[i] - to[i]) * (from[i] - to[i]);
    }
  return sqrt(result);
  }

//---------------------------------------------------------------------------

MFCCVector CDecisionVertex::Direction(MFCCVector from, MFCCVector to)
  {
  if (from.size() != to.size()) ShowMessage("Error in CDecisionVertex::Direction. Vectors with different dimension.");
  MFCCVector result;
  double distance = Distance(from, to);
  for (unsigned int i=0; i<to.size(); i++)
    {
    result.push_back((to[i] - from[i]) / distance);
    }
  return result;
  }

//---------------------------------------------------------------------------

MFCCVectors CDecisionVertex::SpectralTrail(MFCCVectors inputMFCCVectors,int NewSize)
  {
  MFCCVectors result;

  if (inputMFCCVectors.size() == 1)
    {
    for (int i=0; i < NewSize; i++)
      result.push_back(inputMFCCVectors[0]);
    return result;
    }

  //compute length
  double pathlength = 0;
  for (unsigned int i = 1; i < inputMFCCVectors.size(); i++)
    {
    pathlength += Distance(inputMFCCVectors[i-1], inputMFCCVectors[i]);
    }
  //compute new MFCCVectors
  
  result.push_back(inputMFCCVectors[0]);
  double length = pathlength / (NewSize - 1);
  double toNextPoint = length;
  for (unsigned int j = 1; j < inputMFCCVectors.size(); j++)
    {
    while (RoundTo(toNextPoint - Distance(inputMFCCVectors[j - 1],
      inputMFCCVectors[j]), -5) <= 0 )  // ak su dva blizsie ako e-5 tak su totozne
      {
      result.push_back((Direction(inputMFCCVectors[j - 1], inputMFCCVectors[j]) * (toNextPoint)) +
        inputMFCCVectors[j - 1]);
      toNextPoint += length;
      }
     toNextPoint -= Distance(inputMFCCVectors[j - 1], inputMFCCVectors[j]);
    }
  return result;
  }

//---------------------------------------------------------------------------

CDecisionVertex::CDecisionVertex(DecisionClass decisionClass1, DecisionClass decisionClass2, int mfccCount)
{
m_OutputCount = 2;
m_DecisionClass1 = decisionClass1;
m_DecisionClass2 = decisionClass2;
m_MFCCCount = mfccCount;
}

//---------------------------------------------------------------------------

CDecisionVertex::~CDecisionVertex()
   {
   //
   }

//---------------------------------------------------------------------------

MFCCVectors CDecisionVertex::LoadMFCCFromFile (const AnsiString filename,const bool delta,const bool energy)
  {
  MFCCVectors result;
  TStringList* tempList = new TStringList();
  string line;
  ifstream myfile (filename.c_str());
  int i = 0;
  AnsiString s;
  if (myfile.is_open())
  {
    getline (myfile,line); // vynechaj prvy riadok
    getline (myfile,line);
    while ( !myfile.eof() )
      try
      {
      s = line.c_str();
      tempList->DelimitedText = s;
      if (tempList->Strings[1] == "END") return result;

      result.push_back();

      for (int j=1; j < tempList->Count; j++)
        result[i].push_back(StrToFloat(tempList->Strings[j]));

      getline (myfile,line);
      if (delta)
        {
        s = line.c_str();
        tempList->DelimitedText = s;
        for (int j=0; j < tempList->Count; j++)
          result[i].push_back(StrToFloat(tempList->Strings[j]));
        }

      getline (myfile,line);
      if (energy)
        {
        s = line.c_str();
        tempList->DelimitedText = s;
        for (int j=0; j < tempList->Count; j++)
          result[i].push_back(StrToFloat(tempList->Strings[j]));
        }

      getline (myfile,line);
      i++;
      }
      catch (EConvertError &e)
        {
         ShowMessage("Invalid file: " + filename);
        }
    myfile.close();
  }
  else ShowMessage ("Unable to open file " + filename + ".");
  return result;
  }
//---------------------------------------------------------------------------

double CDecisionVertex::GaussianProbability (const double mean, double deviation, double x)
  {
  if (x >0) return (1 - (0.5 * exp(-(x*x) / (2 * deviation * deviation))));
  else if (x <=0) return (0.5 * exp(-(x*x) / (2 * deviation * deviation)));
  return 0;
  }
