//---------------------------------------------------------------------------


#pragma hdrstop

#include "CDTWDecisionVertex.h"
#include <dir.h>
#include "Dialogs.hpp"

//---------------------------------------------------------------------------

#pragma package(smart_init)
//---------------------------------------------------------------------------
CDTWDecisionVertex::CDTWDecisionVertex(DecisionClass decisionClass1, DecisionClass decisionClass2,
  int mfccCount, double maxMFCC, double maxDelta, double maxEnergy)
  :CDecisionVertex(decisionClass1, decisionClass2, mfccCount)
  {
  m_MaxMFCC = maxMFCC;
  m_MaxDelta = maxDelta;
  m_MaxEnergy = maxEnergy;
  for (size_t i = 0; i < decisionClass1.size(); i++)
    LoadSampleSet("MFCC\\" + decisionClass1[i],1);
  for (size_t i = 0; i < decisionClass2.size(); i++)
    LoadSampleSet("MFCC\\" + decisionClass2[i],2);

  }

//---------------------------------------------------------------------------

CDTWDecisionVertex::~CDTWDecisionVertex()
{
m_SampleSet1.clear();
m_SampleSet2.clear();
}

//---------------------------------------------------------------------------


double CDTWDecisionVertex::Distance(const MFCCVector a,const MFCCVector b)
  {
  if (a.size() != b.size())
    {
    ShowMessage("Error in CDTWDecisionVertex::Distance - unequal vector size");
    return MaxDouble;
    }
  double result = 0.;
  for (size_t i = 0; i < a.size(); i++)
    {
    result  += (a[i] - b[i]) * (a[i] - b[i]);  // euklidovka vzdialenost
    }
  return sqrt(result);

  }

//---------------------------------------------------------------------------

double CDTWDecisionVertex::DTWDistance(const MFCCVectors a, const  MFCCVectors b)
  {
  vector < vector<double> >  temp;
  vector < vector<double> > pathLng;
  temp.resize(a.size() + 1);
  pathLng.resize(a.size() + 1);
  for (size_t i = 0; i < temp.size(); i++)
    {
    temp[i].resize(b.size() + 1);
    pathLng[i].resize(b.size() + 1);
    }


  for (size_t i = 1; i <= a.size(); i++)
    {
    temp[i][0] = 10000000000;
    pathLng[i][0] = 0;
    }
  for (size_t i = 1; i <= b.size(); i++)
    {
    temp[0][i] = 10000000000;
    pathLng[0][i] = 0;
    }
  temp[0][0] = 0;
  pathLng[0][0] = 0;
  double dist;
  for (size_t i = 1; i <= a.size(); i++)
    for (size_t j = 1; j <= b.size(); j++)
      {
      dist = Distance (a[i - 1], b[j - 1]);


      if (temp[i-1][j] + dist < temp[i-1][j-1] + dist)
        {
        if (temp[i-1][j] + dist < temp[i][j-1] + dist)
          {
          temp[i][j] = temp[i-1][j] + dist;
          pathLng[i][j] = pathLng[i-1][j] + 1;
          }
        else
          {
          temp[i][j] = temp[i][j-1] + dist;
          pathLng[i][j] = pathLng[i][j-1] + 1;
          }
        }
      else
        {
        if (temp[i-1][j-1] + dist < temp[i][j-1] + dist)
          {
          temp[i][j] = temp[i-1][j-1] + dist;
          pathLng[i][j] = pathLng[i-1][j-1] + 1;
          }

        else
          {
          temp[i][j] = temp[i][j-1] + dist;
          pathLng[i][j] = pathLng[i][j-1] + 1;
          }
        }
      }
  return temp[a.size()][b.size()]/ double(pathLng[a.size()][b.size()] /*a.size() + b.size()*/);
  }
//---------------------------------------------------------------------------

LikelihoodDistribution CDTWDecisionVertex::ComputeLikelihood(MFCCVectors mFCCInput)
  {
  double min1;
  double min2;

  if (m_SampleSet1.size() > 0)
    {
     min1 = DTWDistance(m_SampleSet1[0], mFCCInput);
    for (size_t i = 1; i < m_SampleSet1.size(); i++)
      {
      double temp = DTWDistance(m_SampleSet1[i], mFCCInput);
      if (temp < min1) min1 = temp;
      }
    }
  else ShowMessage("Error in CDTWDecisionVertex::ComputeLikelihood - empty m_SampleSet0");
  if (m_SampleSet2.size() > 0)
    {
    min2 = DTWDistance(m_SampleSet2[0], mFCCInput);
    for (size_t i = 1; i < m_SampleSet2.size(); i++)
      {
      double temp = DTWDistance(m_SampleSet2[i], mFCCInput);
      if (temp < min2) min2 = temp;
      }
    }
  else ShowMessage("Error in CDTWDecisionVertex::ComputeLikelihood - empty m_SampleSet1");

  LikelihoodDistribution result;
  if (min1 < min2)
    {
    result.likelihood1 = 2.;
    result.likelihood2= -2.;
    }
  else
    {
    result.likelihood1 = -2.;
    result.likelihood2 = 2.;
    }
  return result;
  }
//---------------------------------------------------------------------------

void CDTWDecisionVertex::LoadSampleSet(const AnsiString directory,int classIndex, bool delta, bool energy)
  {
  struct ffblk ffblk;
  int done;
  AnsiString s(directory);
  AnsiString s1 = s + "\\*.mfcc";
  done = findfirst(s1.c_str(), &ffblk, 0);
  while (!done)
    {
    s1 = s + "\\" + AnsiString(ffblk.ff_name);
    MFCCVectors temp;
    temp = LoadMFCCFromFile(s1, delta, energy);
    NormalizedMFCCVectors(temp);
    if (classIndex == 1 ) m_SampleSet1.push_back(temp);
    if (classIndex == 2 ) m_SampleSet2.push_back(temp);
    done = findnext(&ffblk);
    }
  }

//---------------------------------------------------------------------------

void CDTWDecisionVertex::NormalizedMFCCVectors(MFCCVectors& mFCCInput)
  {
  for (unsigned int i=0; i< mFCCInput.size(); i++)
    for (unsigned int j=0; j< mFCCInput[i].size(); j++)
      {
      switch (j / m_MFCCCount)
        {
        case 0: mFCCInput[i][j] = mFCCInput[i][j]/ m_MaxMFCC;
        break;
        case 1: mFCCInput[i][j] = mFCCInput[i][j]/m_MaxDelta;
        break;
        default: if (m_MaxEnergy == 0) mFCCInput[i][j] = 0;
           else mFCCInput[i][j] = mFCCInput[i][j]/ m_MaxEnergy;
        }
      }
  }

//---------------------------------------------------------------------------