//---------------------------------------------------------------------------

#ifndef CDecisionVertexH
#define CDecisionVertexH
//---------------------------------------------------------------------------
#include <vector>
#include <System.hpp>
#include <Classes.hpp>
#include <iostream>
#include <fstream>
#include <Math.hpp>
#include <math.h>


using namespace std;

//---------------------------------------------------------------------------

  typedef vector<AnsiString> DecisionClass;

  typedef vector<double> MFCCVector;

  typedef vector< MFCCVector > MFCCVectors;

  struct LikelihoodDistribution
    {
    double likelihood1;
    double likelihood2;
    };

//---------------------------------------------------------------------------

  class CDecisionVertex
    {
     private:
       double Distance(MFCCVector from, MFCCVector to); // vypocita euklidovsku vzdialenost medzi dvomi bodmi
     private:
       MFCCVector Direction(MFCCVector from, MFCCVector to);  // vypocita smernicu
     protected:
       int m_OutputCount;
     protected:
       int m_MFCCCount;  // pocet MFCC parametrov ... ronaky pocet je aj deltaMFCC pripadne deltadeltaMFCC 
     protected:
       DecisionClass m_DecisionClass1;
     protected:
       DecisionClass m_DecisionClass2;
     protected:
       MFCCVectors SpectralTrail(MFCCVectors inputMFCCVectors,int NewSize); // computes desired number of MFCC coeficients
     public:
       CDecisionVertex(DecisionClass decisionClass1, DecisionClass decisionClass2, int mfccCount);
     public:
       CDecisionVertex(){} // data sa nacitaju z filu
     public:
       ~CDecisionVertex();
     public:
       friend MFCCVector operator *(MFCCVector v,double f)
         {
         MFCCVector result;
         for (unsigned int i=0; i < v.size(); i++)
           result.push_back(v[i]*f);
         return result;
         }
     public:
       friend MFCCVector operator +(MFCCVector a,MFCCVector b)
         {
         MFCCVector result;
         for (unsigned int i=0; i < a.size(); i++)
           result.push_back(a[i] + b[i]);
         return result;
         }
     public:
      virtual LikelihoodDistribution ComputeLikelihood(MFCCVectors mFCCInput) = 0;
     public:
      static MFCCVectors LoadMFCCFromFile (const AnsiString filename,const bool delta = true,const bool energy = true);
     public:
      static double GaussianProbability (const double mean, double deviation, double x);
    };

//---------------------------------------------------------------------------
#endif
