//---------------------------------------------------------------------------


#pragma hdrstop

#include "CNeuralNetwork.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)
//---------------------------------------------------------------------------

CNeuralNetwork::CNeuralNetwork(AnsiString filename)
  {
  LoadFromFile(filename);
  }

//---------------------------------------------------------------------------

CNeuralNetwork::CNeuralNetwork(DecisionClass decisionClass1, DecisionClass decisionClass2, int inputCount, int hiddenCount,
  int mfccCount, double maxMFCC, double maxDelta, double maxEnergy):CDecisionVertex(decisionClass1, decisionClass2, mfccCount)
  {
  m_InputCount = inputCount;
  m_HiddenCount = hiddenCount;
  m_MaxMFCC = maxMFCC;
  m_MaxDelta = maxDelta;
  m_MaxEnergy = maxEnergy;
  Randomize();
  for (int i=0;i< m_HiddenCount; i++)
    m_HiddenLayer.push_back(0);

  for (int i=0;i< m_OutputCount; i++)
    m_OutputLayer.push_back(0);

  for (int i=0; i< m_HiddenCount; i++)
    {
    m_HiddenInputWeights.push_back();
    for (int j=0; j < m_InputCount; j++)
      m_HiddenInputWeights[i].push_back(double(random(100000))/100000 - 0.5);
    }

  for (int i=0; i< m_OutputCount; i++)
    {
    m_OutputHiddenWeights.push_back();
    for (int j=0; j< m_HiddenCount; j++)
      m_OutputHiddenWeights[i].push_back(double(random(100000))/100000 - 0.5);
    }
  }

//---------------------------------------------------------------------------
CNeuralNetwork::~CNeuralNetwork()
  {
  m_OutputLayer.clear();
  m_HiddenLayer.clear();

  for (int i=0; i< m_HiddenCount; i++)
    m_HiddenInputWeights[i].clear();
  m_HiddenInputWeights.clear();

  for (int i=0; i< m_OutputCount; i++)
    m_OutputHiddenWeights[i].clear();
  m_OutputHiddenWeights.clear();
 }

//---------------------------------------------------------------------------

void CNeuralNetwork::Evaluate(vector <double> input)
  {
  for (int i=0; i< m_HiddenCount; i++)
    {
    m_HiddenLayer[i] = 0;
    for (int j=0; j < m_InputCount; j++)
      m_HiddenLayer[i] += input[j] * m_HiddenInputWeights[i][j];
    ActivationFunction(m_HiddenLayer[i]);
    }

  for (int i=0; i< m_OutputCount; i++)
    {
    m_OutputLayer[i] = 0;
    for (int j=0; j< m_HiddenCount; j++)
      m_OutputLayer[i] = m_HiddenLayer[j]*m_OutputHiddenWeights[i][j];
    ActivationFunction(m_OutputLayer[i]);
    }
  }
//---------------------------------------------------------------------------

LikelihoodDistribution CNeuralNetwork::ComputeLikelihood(MFCCVectors mFCCInput)
  {
  // prerobim vstup na vstup pre evaluate
  vector <double> temp1 = NormalizedMFCCVectors(mFCCInput);
  Evaluate(temp1);
  LikelihoodDistribution result;
  //result.likelihood1.decisionClass = m_DecisionClass1;
  //result.likelihood2.decisionClass = m_DecisionClass2;

  double dif =  m_OutputLayer[0] - m_OutputLayer[1] ;
  result.likelihood1 = dif;
  result.likelihood2 = - dif;
  /*if (dif >= 1)
    if ((m_OutputLayer[0] > 0.5) && (m_OutputLayer[1] < -0.5))
      {
      result.likelihood1 = 1;
      result.likelihood2 = 0;
      }
    else
      {
      result.likelihood1 = 1;
      result.likelihood2 = 0;
      }
  else if (dif <= -1)
    if ((m_OutputLayer[0] > 0.5) && (m_OutputLayer[1] < -0.5))
      {
      result.likelihood1 = 0;
      result.likelihood2 = 1;
      }
    else
      {
      result.likelihood1 = 0;
      result.likelihood2 = 1;
      }
   else
      {
      double temp2 = RoundTo(CDecisionVertex::GaussianProbability(0, 0.55,  dif), -3);
      result.likelihood1 = temp2;
      result.likelihood2 = 1 - temp2; 
      }    */

  return result;
  }
//---------------------------------------------------------------------------

void CNeuralNetwork::LoadFromFile (AnsiString filename)
  {
  string line;
  ifstream myfile (filename.c_str());
  if (myfile.is_open())
    {
    getline (myfile, line);
    getline (myfile, line);
    m_MaxMFCC = atoi (line.c_str());
    getline (myfile, line);
    m_MaxDelta = atoi (line.c_str());
    getline (myfile, line);
    m_MaxEnergy = atoi (line.c_str());
    getline (myfile, line);
    m_MFCCCount = atoi (line.c_str());
    getline (myfile, line);
    m_OutputCount = atoi (line.c_str());     //OutputCount

    for (int i=0;i< m_OutputCount; i++)
      m_OutputLayer.push_back(0);
    
    getline (myfile,line);
    if (line != "#firstDecisionClass") throw ("File corupted!");

    while (line != "#secondDecisionClass")  // load decision Class1
      {
      getline (myfile,line);

      AnsiString s(line.c_str());
      m_DecisionClass1.push_back(s);
      }

    while (line != "#outputhiddenweights")  // load decision Class2
      {
      getline (myfile, line);
      AnsiString s(line.c_str());
      m_DecisionClass2.push_back(s);
      }


    getline (myfile,line);
    m_HiddenCount = atoi (line.c_str());     //HiddenCount
    for (int i=0;i< m_HiddenCount; i++)
      m_HiddenLayer.push_back(0);



    for (int i=0; i< m_OutputCount; i++)   // load output hidden weights
      {
      m_OutputHiddenWeights.push_back();
      for (int j=0; j< m_HiddenCount; j++)
        {
        getline (myfile,line);
        m_OutputHiddenWeights[i].push_back(atof(line.c_str()));
        }
      }

    getline (myfile,line);           // check integrity
    if (line != "#hiddeninputweights") throw ("File corupted!");

    getline (myfile,line);     // InputCount
    m_InputCount = atoi(line.c_str());

    for (int i=0; i< m_HiddenCount; i++)  // load hidden input weights
      {
      m_HiddenInputWeights.push_back();
      for (int j=0; j< m_InputCount; j++)
        {
        getline (myfile,line);
        m_HiddenInputWeights[i].push_back(atof(line.c_str()));
        }
      }
    myfile.close();
    }
  else throw ("ERROR");
}

//---------------------------------------------------------------------------

vector<double> CNeuralNetwork::NormalizedMFCCVectors(const MFCCVectors mFCCInput)
  {
  vector <double> temp;
  MFCCVectors tempMFCCVectors =  SpectralTrail(mFCCInput, (m_InputCount / mFCCInput[0].size()));
  for (unsigned int i=0; i< tempMFCCVectors.size(); i++)
    for (unsigned int j=0; j< tempMFCCVectors[i].size(); j++)
      {
      switch (j / m_MFCCCount)
        {
        case 0: temp.push_back(Normalize(tempMFCCVectors[i][j], m_MaxMFCC));
        break;
        case 1: temp.push_back(Normalize(tempMFCCVectors[i][j], m_MaxDelta));
        break;
        default: temp.push_back(Normalize(tempMFCCVectors[i][j], m_MaxEnergy));
        }
      }
  return temp;
  }

//---------------------------------------------------------------------------
