
//---------------------------------------------------------------------------

#ifndef CNeuralNetworkH
#define CNeuralNetworkH

#include "CDecisionVertex.h"
#include <math>
#include <system.hpp>

//Neuronova siet s plnym prepojenim a jednou skrytou vrstvou
class CNeuralNetwork: public CDecisionVertex
  {
  protected:
    double m_MaxMFCC;
  protected:
    double m_MaxDelta;
  protected:
    double m_MaxEnergy;
  protected:
    int m_HiddenCount;
  protected:
    int m_InputCount;
  protected:
    vector<double> m_OutputLayer; // hodnoty vystupnej vrstvy
  protected:
    vector<double> m_HiddenLayer; //hodnoty skrytej vrstvy ... pouzije sa pri uceni
  protected:
    vector < vector <double> > m_HiddenInputWeights; // [i][j] => ita hidden jta input
  protected:
    vector < vector <double> > m_OutputHiddenWeights; // [i][j] => ita output jta hidden
  protected:
    inline void ActivationFunction(double& net)
      {
      net = 2/(1+exp(-net)) -1;
      } // pouzijem bipolarnu sigmoidu ako aktivacnu fciu
  protected:
    inline double Normalize(double value, double maxValue)
      {
      if (maxValue == 0)  return 0;
      double result = value  /  maxValue;
      if (result > 1) return 1;
      if (result < -1) return -1;
      return result;
      }// vrati normalizovanu hodnotu tj. z intervalu <-1,1>
  protected:
    void Evaluate(vector <double> input); // vypocet vystupu neurvonove siete ocakava vstup z intervalu <-1,1>
  public:
    CNeuralNetwork(DecisionClass decisionClass1, DecisionClass decisionClass2, int inputCount, int hiddenCount,
      int mfccCount, double maxMFCC, double maxDelta, double maxEnergy);
  public:
    CNeuralNetwork(AnsiString filename);
  public:
    ~CNeuralNetwork();
  public:
    LikelihoodDistribution ComputeLikelihood(MFCCVectors mFCCInput);  // upravi vstup pre Evaluate a zavola; na vystup zase vypocita distribuciu pravdepodobnosti

 //------------------!!!interpretacia hodnot v *.nne subore!!!----------------
 //v kazdom riadku je jedna hodnota
 //vyznam je naslednovny:
 //uspesnost natrenovanej NN
 //maxMFCC // maximalna hodnota MFCC koeficientov
 //maxDelta
 //maxEnergia
 //MFCCCount - pocet MFCC koeficientov vo vstupnom vektore
 //OutputCount // pocet neuronov na vystupnej vrstve
 //#firstDecisionClass // oznacuje zaciatok prvej triedy            }
 //lubovolny pocet prvkov
 //#secondDecisionClass
 //lubovolny pocet prvkov
 //#outputhiddenweights // oznacuje zaciatok nacitavania outputHiddenWeights
 // HiddenCount // pocet neuronov na skrytej vrstve
 // hodnoty OutpuHiddenWeights[i][j]  kde i ide po OutputCount a j po HiddenCount
 //#hiddeninputweights // oznacuje zaciatok nacitavania hiddenOutputWeights
 // InputCount // pocet neuronov na skrytej vrstve
 // hodnoty HiddenOutputWeights[i][j]  kde i ide po HiddenCount a j po InputCount
 //---------------------------------------------------------------------------
 public:
   void LoadFromFile (AnsiString filename);

 public:
   vector<double> NormalizedMFCCVectors(const MFCCVectors mFCCInput); // znormalizuje vectory do <-1,1> a upravy na velkost pomocou spektralnej cesty
  };
//---------------------------------------------------------------------------
#endif
