//---------------------------------------------------------------------------


#pragma hdrstop

#include "CTrainableNeuralNetwork.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

CTrainableNeuralNetwork::CTrainableNeuralNetwork(DecisionClass decisionClass1, DecisionClass decisionClass2, int inputCount,
  int hiddenCount, int mfccCount, double maxMFCC, double maxDelta, double maxEnergy)
  :CNeuralNetwork(decisionClass1, decisionClass2, inputCount, hiddenCount, mfccCount, maxMFCC, maxDelta, maxEnergy)
  {
  randomize();
  for (unsigned int j=0; j< m_DecisionClass1.size() ;j++)
    {
    LoadSampleSet("MFCC\\" + UpperCase(m_DecisionClass1[j]), 0);
    }

  for (unsigned int j=0; j< m_DecisionClass2.size() ;j++)
    {
    LoadSampleSet("MFCC\\" + UpperCase(m_DecisionClass2[j]), 1);
    }
  }
//---------------------------------------------------------------------------

CTrainableNeuralNetwork::~CTrainableNeuralNetwork()
  {
  for (unsigned int i=0; i<m_SampleSet0.size(); i++)
    m_SampleSet0[i].input.clear();

  for (unsigned int i=0; i<m_SampleSet1.size(); i++)
    m_SampleSet1[i].input.clear();

  for (unsigned int i=0; i<m_TrainingSet.size(); i++)
    m_TrainingSet[i].input.clear();

  for (unsigned int i=0; i<m_TestingSet.size(); i++)
    m_TestingSet[i].input.clear();
  }

//---------------------------------------------------------------------------

double CTrainableNeuralNetwork::ComputeError(vector<CSample> sampleSet)
  {
  double result = 0;
  for (unsigned int i=0; i < sampleSet.size(); i++)
    {
    double tempError = 0;
    Evaluate(sampleSet[i].input);
    for (int j = 0; j < m_OutputCount; j++)
      if (j == sampleSet[i].classIndex) tempError += (1 - m_OutputLayer[j]) *(1 - m_OutputLayer[j]);
      else tempError += (-1 - m_OutputLayer[j]) * (-1 - m_OutputLayer[j]);
    result += 0.5 * tempError;
    }
  return result;
  }

//---------------------------------------------------------------------------

double CTrainableNeuralNetwork::ErrorInPercent(vector<CSample> sampleSet, double errorToleration)
  {
  double result = 0;
  for (unsigned int i=0; i < sampleSet.size(); i++)
    {
    bool Ok = true;
    Evaluate(sampleSet[i].input);
    for (int j = 0; j < m_OutputCount; j++)
      if (j == sampleSet[i].classIndex) Ok = Ok && (1 - m_OutputLayer[j]) <= errorToleration;
      else  Ok = Ok && (-1 - m_OutputLayer[j]) <= errorToleration;

    if (!Ok) result += 1;
    }
  
  return (result / sampleSet.size()) * 100;
  }

//---------------------------------------------------------------------------

void CTrainableNeuralNetwork::Train(int epochCount, double alpha)
  {
  vector<double> errorInHiddenLayer;
  vector<double> errorInOutputLayer;
  for (int i=0; i < epochCount ;i++)
    {
    random_shuffle(m_TrainingSet.begin(), m_TrainingSet.end()); // nahodne zorad prvky mnoziny
    for (unsigned int j=0; j < m_TrainingSet.size() ; j++)
      {
      errorInHiddenLayer.clear();
      errorInOutputLayer.clear();

      Evaluate(m_TrainingSet[j].input); // vypocitaj vystup siete

      for (int k=0; k < m_OutputCount; k++)  //chyba na vystupnej vrstve
        {
        if (k == m_TrainingSet[j].classIndex) errorInOutputLayer.push_back(
          (1 - m_OutputLayer[k]) * DerActivationFuction(m_OutputLayer[k]));   // ma to byt one hot
        else errorInOutputLayer.push_back(
          (-1 - m_OutputLayer[k]) * DerActivationFuction(m_OutputLayer[k]));
        } // for

      for (int l = 0; l < m_HiddenCount; l++) // chyba na skrytej vrstve
        {
        double temp = 0;
        for (int k=0; k < m_OutputCount; k++)
          temp += errorInOutputLayer[k] * m_OutputHiddenWeights[k][l];

        errorInHiddenLayer.push_back(temp * DerActivationFuction(m_HiddenLayer[l]));
        } // for

      for (int k = 0; k < m_HiddenCount; k++)  //uprava vah medzi hidden a input vrstovou
        {
        for (int l=0; l < m_InputCount; l++)
          m_HiddenInputWeights[k][l] += alpha * errorInHiddenLayer[k]* m_TrainingSet[j].input[l];
        } // for

      for (int k = 0; k < m_OutputCount; k++) // uprava vah medzi hidden a output vrstvou
        {
        for (int l=0; l < m_HiddenCount; l++)
          m_OutputHiddenWeights[k][l] += alpha * errorInOutputLayer[k]* m_HiddenLayer[l];
        } // for
      } // for one sample of training set
    } // main for
  } //function

//---------------------------------------------------------------------------

void CTrainableNeuralNetwork::MakeTrainigAndTestingSet(unsigned int percentInTrainingSet)
  {
  random_shuffle(m_SampleSet0.begin(), m_SampleSet0.end());
  random_shuffle(m_SampleSet1.begin(), m_SampleSet1.end());

  m_TestingSet.clear();
  m_TrainingSet.clear();
  for (unsigned int i=0; i < m_SampleSet0.size() ;i++)
    {
    if ( (float(i) / m_SampleSet0.size()) <= (float(percentInTrainingSet) / 100)) m_TrainingSet.push_back(m_SampleSet0[i]);
    else m_TestingSet.push_back(m_SampleSet0[i]);
    }

  for (unsigned int i=0; i < m_SampleSet1.size() ;i++)
    {
    if ( (float(i) / m_SampleSet1.size()) <= (float(percentInTrainingSet) / 100)) m_TrainingSet.push_back(m_SampleSet1[i]);
    else m_TestingSet.push_back(m_SampleSet1[i]);
    }
  }

//---------------------------------------------------------------------------

void CTrainableNeuralNetwork::LoadSampleSet(const AnsiString directory,int classIndex, bool delta, bool energy)
  {
  struct ffblk ffblk;
  int done;
  AnsiString s(directory);
  AnsiString s1 = s + "\\*.mfcc";
  done = findfirst(s1.c_str(), &ffblk, 0);
  while (!done)
    {
    s1 = s + "\\" + AnsiString(ffblk.ff_name);
    CSample temp;
    temp.classIndex = classIndex;
    temp.input = NormalizedMFCCVectors(LoadMFCCFromFile(s1, delta, energy));
    if (classIndex == 0 ) m_SampleSet0.push_back(temp);
    if (classIndex == 1 ) m_SampleSet1.push_back(temp);
    done = findnext(&ffblk);
    }
  }

//---------------------------------------------------------------------------

void CTrainableNeuralNetwork::ExportToFile(const AnsiString filename)
  {
  FILE *stream;

  stream = fopen(filename.c_str(), "w");

  fprintf(stream, FloatToStr(ErrorOnTestingSet()).c_str());
  fprintf(stream, "\n");
  
  fprintf(stream, FloatToStr(m_MaxMFCC).c_str());
  fprintf(stream, "\n");

  fprintf(stream, FloatToStr(m_MaxDelta).c_str());
  fprintf(stream, "\n");

  fprintf(stream, FloatToStr(m_MaxEnergy).c_str());
  fprintf(stream, "\n");

  fprintf(stream, IntToStr(m_MFCCCount).c_str());
  fprintf(stream, "\n");

  fprintf(stream, IntToStr(m_OutputCount).c_str());
  fprintf(stream, "\n");

  fprintf(stream, "#firstDecisionClass");
  fprintf(stream, "\n");
  for (unsigned int j=0; j < m_DecisionClass1.size();j++)
    {
    fprintf(stream, m_DecisionClass1[j].c_str());
    fprintf(stream, "\n");
    }

  fprintf(stream, "#secondDecisionClass");
  fprintf(stream, "\n");
  for (unsigned int j=0; j < m_DecisionClass2.size();j++)
    {
    fprintf(stream, m_DecisionClass2[j].c_str());
    fprintf(stream, "\n");
    }


  fprintf(stream, "#outputhiddenweights");
  fprintf(stream, "\n");
  fprintf(stream, IntToStr(m_HiddenCount).c_str()); //HiddemCount
  fprintf(stream, "\n");

  for (int i=0; i< m_OutputCount; i++)   // load output hidden weights
    {
    for (int j=0; j< m_HiddenCount; j++)
      {
      AnsiString s(m_OutputHiddenWeights[i][j]);
      fprintf(stream, s.c_str());
      fprintf(stream, "\n");
      }
    }


  fprintf(stream, "#hiddeninputweights");
  fprintf(stream, "\n");
  fprintf(stream, IntToStr(m_InputCount).c_str()); //HiddenCount
  fprintf(stream, "\n");

  for (int i=0; i< m_HiddenCount; i++)  // load hidden input weights
    {
    for (int j=0; j< m_InputCount; j++)
      {
      AnsiString s(m_HiddenInputWeights[i][j]);
      fprintf(stream, s.c_str());
      fprintf(stream, "\n");
      }
    }

  fclose(stream);
  }


//---------------------------------------------------------------------------

void CTrainableNeuralNetwork::ShowSampleSet(TStrings* lines)
{
AnsiString s;
for (unsigned int i=0; i < m_SampleSet0.size(); i++)
  {
  s = "Sample " + IntToStr(i) + ", class " + IntToStr(m_SampleSet0[i].classIndex) + "  ";
  lines->Add(s);
  s = "";
  for (unsigned int j=0; j < m_SampleSet0[i].input.size();j++)
    {
    s += FloatToStr(RoundTo(m_SampleSet0[i].input[j], -3));
    s += ", ";
    }
  lines->Add(s);
  lines->Add("");
  }

for (unsigned int i=0; i < m_SampleSet1.size(); i++)
  {
  s = "Sample " + IntToStr(i) + ", class " + IntToStr(m_SampleSet1[i].classIndex) + "  ";
  lines->Add(s);
  s = "";
  for (unsigned int j=0; j < m_SampleSet1[i].input.size();j++)
    {
    s += FloatToStr(RoundTo(m_SampleSet1[i].input[j], -3));
    s += ", ";
    }
  lines->Add(s);
  lines->Add("");
  }

}

//---------------------------------------------------------------------------

AnsiString CTrainableNeuralNetwork::GetDecisionClass1()
{
AnsiString s;
  for (unsigned int j=0; j < m_DecisionClass1.size();j++)
    {
    s += m_DecisionClass1[j];
    }
  return s;
}

//---------------------------------------------------------------------------

AnsiString CTrainableNeuralNetwork::GetDecisionClass2()
{
  AnsiString s;
  for (unsigned int j=0; j < m_DecisionClass2.size();j++)
    {
    s += m_DecisionClass2[j];
    }
  return s;
}

//---------------------------------------------------------------------------

void CTrainableNeuralNetwork::ClearSampleSet()
  {
  m_SampleSet0.clear();
  m_SampleSet1.clear();
  m_TrainingSet.clear();
  m_TestingSet.clear();
  }

//---------------------------------------------------------------------------

void CTrainableNeuralNetwork::ResetWeights()
  {
  for (int i=0; i< m_HiddenCount; i++)
    {
    for (int j=0; j < m_InputCount; j++)
      m_HiddenInputWeights[i][j] = (double(random(60000))/100000.) - 0.3;
    }

  for (int i=0; i< m_OutputCount; i++)
    {
    for (int j=0; j< m_HiddenCount; j++)
      m_OutputHiddenWeights[i][j] = (double(random(60000))/100000.) - 0.3;
    }
  }
//---------------------------------------------------------------------------

AnsiString CTrainableNeuralNetwork::GetNNParameters()
  {
  AnsiString s;
  s = GetDecisionClass1();
  s += ' ';
  s += GetDecisionClass2();
  s += ' ';
  s +=   IntToStr(m_InputCount);
  s += ' ';
  s +=   IntToStr(m_HiddenCount);
  s += ' ';
  s +=   FloatToStr(m_MaxMFCC);
  s += ' ';
  s +=   FloatToStr(m_MaxDelta);
  s += ' ';
  s +=   FloatToStr(m_MaxEnergy);

  return s;
  }

//---------------------------------------------------------------------------

