//---------------------------------------------------------------------------

#ifndef CTrainableNeuralNetworkH
#define CTrainableNeuralNetworkH
#include "CNeuralNetwork.h"
#include <algorithm>
#include <Classes.hpp>
#include <dir.h> 

struct CSample
  {
  vector<double> input; // vstup do neuronovej siete
  int classIndex; // index triedy do ktorej patri   !!!pocitanie zacina od nuly!!!
  };

class CTrainableNeuralNetwork: public CNeuralNetwork
  {
  private:
    vector<CSample> m_SampleSet0;
  private:
    vector<CSample> m_SampleSet1;
  private:
    vector<CSample> m_TrainingSet;
  private:
    vector<CSample> m_TestingSet;
  private:
    inline double DerActivationFuction(double value) // derivacia aktivacnej funkcie
      {
      return 0.5 * (1 - value * value);
      }
  private:
    double ComputeError(vector<CSample> sampleSet);   // vracia kumulovanu chybu cez vsetky vzorky testing
  private:
    double ErrorInPercent(vector<CSample> sampleSet, double errorToleration);


  public:
    CTrainableNeuralNetwork(DecisionClass decisionClass1, DecisionClass decisionClass2, int inputCount, int hiddenCount,
      int mfccCount, double maxMFCC, double maxDelta, double maxEnergy);

  public:
    ~CTrainableNeuralNetwork();

  public:
    void Train(int epochCount, double alpha);

  public:
    double ErrorOnTrainingSet()
      {
      return ComputeError(m_TrainingSet);
      }
  public:
    double ErrorOnTestingSet()
      {
      return ComputeError(m_TestingSet);
      }
  public:
    double ErrorInPercentOnTrainingSet(double errorToleration)
      {
      return ErrorInPercent(m_TrainingSet, errorToleration);
      }
  public:
    double ErrorInPercentOnTestingSet(double errorToleration)
      {
      return ErrorInPercent(m_TestingSet, errorToleration);
      }
      
  public:
    void MakeTrainigAndTestingSet(unsigned int percentInTrainingSet); // rozdeli sample set na training set a testing set
  public:
    void LoadSampleSet(const AnsiString directory,int classIndex, bool delta = true, bool energy = true);
  public:
    void ExportToFile(const AnsiString filename);
  public:
    void ShowSampleSet(TStrings* lines);
  public:
    AnsiString GetDecisionClass1 ();
  public:
    AnsiString GetDecisionClass2 ();
  public:
    void ClearSampleSet();
  public:
    void ResetWeights();
  public:
    unsigned int GetSampleSetSize()
      {
      return m_SampleSet0.size() + m_SampleSet1.size();
      }
  public:
    AnsiString GetNNParameters();
  };
//---------------------------------------------------------------------------
#endif
